@@ -4,7 +4,7 @@
  * Gnome Crystal
  * document.cc 
  *
- * Copyright (C) 2000-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2000-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -41,28 +41,21 @@
 #include <glade/glade.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
-//#include <libgnomevfs/gnome-vfs-file-info.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <vector>
-#include <map>
-#ifdef HAVE_FSTREAM
-#	include <fstream>
-#else
-#	include <fstream.h>
-#endif
-#ifdef HAVE_OSTREAM
-#	include <ostream>
-#else
-#	include <ostream.h>
-#endif
-#ifdef HAVE_SSTREAM
-#	include <sstream>
-#else
-#	include <sstream.h>
-#endif
-#include <glib/gi18n.h>
 #include <clocale>
 #include <cmath>
+#include <vector>
+#include <map>
+#include <fstream>
+#include <ostream>
+#include <sstream>
+#include <gio/gio.h>
+#include <glib/gi18n.h>
+#ifdef HAVE_OPENBABEL_2_2
+#	include <openbabel/format.h>
+#	include <openbabel/obconversion.h>
+#	include <openbabel/math/matrix3x3.h>
+using namespace OpenBabel;
+#endif
 #include <cstring>
 
 #define SAVE	1
@@ -81,12 +74,15 @@ gcDocument::gcDocument (gcApplication *pApp) :CrystalDoc (pApp)
 	m_title = NULL;
 	m_bClosing = false;
 	m_ReadOnly = false;
+	m_Author = m_Mail = m_Label = m_Comment = NULL;
+	g_date_set_time_t (&m_CreationDate, time (NULL));
+	g_date_clear (&m_RevisionDate, 1);
 }
 
 gcDocument::~gcDocument()
 {
-	if (m_filename != NULL) g_free(m_filename);
-	if (m_title) g_free(m_title);
+	g_free(m_filename);
+	g_free(m_title);
 	Reinit();
 	Dialog *dialog;
 	while (!m_Dialogs.empty())
@@ -174,10 +170,20 @@ void gcDocument::SetCell(gcLattices lattice, gdouble a, gdouble b, gdouble c, gd
 
 void gcDocument::SetFileName (const string &filename)
 {
-	GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
-	gnome_vfs_get_file_info (filename.c_str (), info, GNOME_VFS_FILE_INFO_DEFAULT);
-	m_ReadOnly = !(info->permissions & (GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_GROUP_WRITE));
-	gnome_vfs_file_info_unref (info);
+	GFile *file = g_file_new_for_uri (filename.c_str ());
+	GError *error = NULL;
+	GFileInfo *info = g_file_query_info (file,
+							  G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
+							  G_FILE_QUERY_INFO_NONE, NULL, &error);
+	if (error) {
+		g_warning ("GIO error: %s", error->message);
+		g_error_free (error);
+		m_ReadOnly = true;
+	} else
+		m_ReadOnly = !g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
+	if (info)
+		g_object_unref (info);
+	g_object_unref (file);
 	if (m_filename)
 		g_free (m_filename);
 	m_filename = g_strdup (filename.c_str ());
@@ -196,18 +202,34 @@ void gcDocument::SetFileName (const string &filename)
 	int j = filename.length () - 1;
 	while ((i < j) && (m_filename[j] != '.'))
 		j--;
-	gchar* title = (strcmp (m_filename + j, ".gcrystal"))? g_strdup (m_filename + i):g_strndup (m_filename + i, j - i);
-	SetTitle (title);
-	g_free (title);
+	if (!m_title) {
+		g_free (m_Label);
+		char *buf = (strcmp (m_filename + j, ".gcrystal"))? g_strdup (m_filename + i): g_strndup (m_filename + i, j - i);
+		m_Label = g_uri_unescape_string (buf, NULL);
+		g_free (buf);
+	}
 }
 
 void gcDocument::SetTitle(const gchar* title)
 {
-	if (m_title) g_free(m_title);
+	g_free(m_title);
 	m_title = g_strdup(title);
+	g_free(m_Label);
+	m_Label = g_strdup(title);	
 }
 
-void gcDocument::Save()
+static int cb_xml_to_vfs (GOutputStream *output, const char* buf, int nb)
+{
+	GError *error = NULL;
+	int n = g_output_stream_write (output, buf, nb, NULL, &error);
+	if (error) {
+		g_message ("GIO error: %s", error->message);
+		g_error_free (error);
+	}
+	return n;
+}
+
+void gcDocument::Save() const
 {
 	if (!m_filename)
 		return;
@@ -216,12 +238,77 @@ void gcDocument::Save()
 	try {
 		xml = BuildXMLTree();
 
-		if (xmlSaveFile (m_filename, xml) < 0)
-			Error (SAVE);
-			
+		if (!g_date_valid (&m_CreationDate))
+			g_date_set_time_t (&const_cast <gcDocument *> (this)->m_CreationDate, time (NULL));
+		g_date_set_time_t (&const_cast <gcDocument *> (this)->m_RevisionDate, time (NULL));
+		gchar tmp[64];
+		g_date_strftime (tmp, sizeof (tmp), "%m/%d/%Y", &m_CreationDate);
+		xmlNewProp (xml->children, (xmlChar*) "creation", (xmlChar*) tmp);
+		g_date_strftime (tmp, sizeof (tmp), "%m/%d/%Y", &m_RevisionDate);
+		xmlNewProp (xml->children, (xmlChar*) "revision", (xmlChar*) tmp);
+		xmlNodePtr node;
+
+		if (m_title && *m_title) {
+			node = xmlNewDocNode (xml, NULL, (xmlChar*) "title", (xmlChar*) m_title);
+			if (node)
+				xmlAddChild (xml->children, node);
+			else
+				throw (int) 0;
+		}
+		if ((m_Author && *m_Author) || (m_Mail && *m_Mail)) {
+			node = xmlNewDocNode (xml, NULL, (xmlChar*) "author", NULL);
+			if (node) {
+				if (m_Author && *m_Author)
+					xmlNewProp (node, (xmlChar*) "name", (xmlChar*) m_Author);
+				if (m_Mail && *m_Mail)
+					xmlNewProp (node, (xmlChar*) "e-mail", (xmlChar*) m_Mail);
+				xmlAddChild (xml->children, node);
+			}
+			else
+				throw (int) 0;
+		}
+		if (m_Comment && *m_Comment) {
+			node = xmlNewDocNode (xml, NULL, (xmlChar*) "comment", (xmlChar*) m_Comment);
+			if (node)
+				xmlAddChild (xml->children, node);
+			else
+				throw (int) 0;
+		}
+		xmlIndentTreeOutput = true;
+		xmlKeepBlanksDefault (0);
+	
+		xmlOutputBufferPtr buf = xmlAllocOutputBuffer (NULL);
+		GFile *file = g_file_new_for_uri (m_filename);
+		GError *error = NULL;
+			if (g_file_query_exists (file, NULL)) {
+				// FIXME: for now, delete it, but we might make a backup?
+				g_file_delete (file, NULL, &error);
+				if (error) {
+					g_message ("GIO error: %s", error->message);
+					g_error_free (error);
+					g_object_unref (file);
+					throw (int) 1;
+				}
+			}
+		GOutputStream *output = G_OUTPUT_STREAM (g_file_create (file, G_FILE_CREATE_NONE, NULL, &error));
+		if (error) {
+			g_message ("GIO error: %s", error->message);
+			g_error_free (error);
+			g_object_unref (file);
+			throw (int) 1;
+		}
+		buf->context = output;
+		buf->closecallback = NULL;
+		buf->writecallback = (xmlOutputWriteCallback) cb_xml_to_vfs;
+		int n = xmlSaveFormatFileTo (buf, xml, NULL, true);
+		g_output_stream_close (output, NULL, NULL);
+		g_object_unref (file);
+		if (n < 0)
+			throw 1;
+
 		xmlFreeDoc (xml);
-		SetDirty (false);
-		m_ReadOnly = false;	// if saving succeded, the file is not read only...
+		const_cast <gcDocument *> (this)->SetDirty (false);
+		const_cast <gcDocument *> (this)->m_ReadOnly = false;	// if saving succeded, the file is not read only...
 	}
 	catch (int num) {
 		xmlFreeDoc (xml);
@@ -229,24 +316,26 @@ void gcDocument::Save()
 	}
 }
 
-void gcDocument::Error (int num)
+void gcDocument::Error (int num) const
 {
 	gchar *mess = NULL;
 	GtkWidget* message;
+	char *unescaped = g_uri_unescape_string (m_filename, NULL);
 	switch (num) {
 	case SAVE:
-		mess = g_strdup_printf (_("Could not save file\n%s"), m_filename);
+		mess = g_strdup_printf (_("Could not save file\n%s"), unescaped);
 		break;
 	case LOAD:
-		mess = g_strdup_printf (_("Could not load file\n%s"), m_filename);
+		mess = g_strdup_printf (_("Could not load file\n%s"), unescaped);
 		break;
 	case XML:
-		mess = g_strdup_printf (_("%s: invalid xml file.\nTree is empty?"), m_filename);
+		mess = g_strdup_printf (_("%s: invalid xml file.\nTree is empty?"), unescaped);
 		break;
 	case FORMAT:
-		mess = g_strdup_printf (_("%s: invalid file format."), m_filename);
+		mess = g_strdup_printf (_("%s: invalid file format."), unescaped);
 		break;
 	}
+	g_free (unescaped);
 	message = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess);
 	g_signal_connect_swapped (G_OBJECT (message), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (message));
 	gtk_widget_show (message);
@@ -262,14 +351,14 @@ bool gcDocument::Load (const string &filename)
 	else oldfilename = NULL;
 	oldtitle = g_strdup (m_title);
 	try {
-		if (SetFileName (filename), !m_filename || !m_title)
-			throw (int) 0;
-		if (!(xml = xmlParseFile (filename.c_str ())))
+		if (SetFileName (filename), !m_filename || !m_Label)
 			throw (int) 1;
-		if (xml->children == NULL)
+		if (!(xml = xmlParseFile (filename.c_str ())))
 			throw (int) 2;
-		if (strcmp ((char*) xml->children->name, "crystal"))
+		if (xml->children == NULL)
 			throw (int) 3;
+		if (strcmp ((char*) xml->children->name, "crystal"))
+			throw (int) 4;
 		if (oldfilename)
 			g_free(oldfilename);
 		g_free (oldtitle);
@@ -281,9 +370,10 @@ bool gcDocument::Load (const string &filename)
 		switch (num)
 		{
 		case 2:
+		case 3:
 			Error(XML);
 			break;
-		case 3:
+		case 4:
 			Error(FORMAT);
 			break;
 		default:
@@ -318,6 +408,65 @@ void gcDocument::ParseXMLTree(xmlNode* xml)
 	//look for generator node
 	unsigned version = 0xffffff , major, minor, micro;
 	node = xml->children;
+	if (m_Author) {
+		g_free (m_Author);
+		m_Author = NULL;
+	}
+	if (m_Mail) {
+		g_free (m_Mail);
+		m_Mail = NULL;
+	}
+	if (m_Comment) {
+		g_free (m_Comment);
+		m_Comment = NULL;
+	}
+	g_date_clear (&m_CreationDate, 1);
+	g_date_clear (&m_RevisionDate, 1);
+	txt = (char*) xmlGetProp (xml, (xmlChar*) "creation");
+	if (txt) {
+		g_date_set_parse (&m_CreationDate, txt);
+		if (!g_date_valid (&m_CreationDate))
+			g_date_clear (&m_CreationDate, 1);
+		xmlFree (txt);
+	}
+	txt = (char*) xmlGetProp (xml, (xmlChar*) "revision");
+	if (txt) {
+		g_date_set_parse (&m_RevisionDate, txt);
+		if (!g_date_valid(&m_RevisionDate))
+			g_date_clear(&m_RevisionDate, 1);
+		xmlFree (txt);
+	}
+
+	node = GetNodeByName (xml, "title");
+	if (node) {
+		txt = (char*) xmlNodeGetContent (node);
+		if (txt) {
+			g_free (m_title);
+			m_title = g_strdup (txt);
+			xmlFree (txt);
+		}
+	}
+	node = GetNodeByName (xml, "author");
+	if (node) {
+		txt = (char*) xmlGetProp (node, (xmlChar*) "name");
+		if (txt) {
+			m_Author = g_strdup (txt);
+			xmlFree (txt);
+		}
+		txt = (char*) xmlGetProp (node, (xmlChar*) "e-mail");
+		if (txt) {
+			m_Mail = g_strdup (txt);
+			xmlFree (txt);
+		}
+	}
+	node = GetNodeByName (xml, "comment");
+	if (node) {
+		txt = (char*) xmlNodeGetContent (node);
+		if (txt) {
+			m_Comment = g_strdup (txt);
+			xmlFree (txt);
+		}
+	}
 	while (node)
 	{
 		if (!strcmp ((const char*)(node->name), "generator")) break;
@@ -422,27 +571,31 @@ void gcDocument::OnNewDocument()
 typedef struct {int n; std::list<CrystalAtom*> l;} sAtom;
 typedef struct {int n; std::list<CrystalLine*> l;} sLine;
 
-void gcDocument::OnExportVRML (const string &FileName)
+void gcDocument::OnExportVRML (const string &FileName) const
 {
 	char *old_num_locale, tmp[128];
 	double x0, x1, x2, x3, x4, x5;
 	int n = 0;
 	try {
 		ostringstream file;
-		GnomeVFSHandle *handle = NULL;
-		GnomeVFSFileSize fs;
-		GnomeVFSResult res;
+		GError *error = NULL;
+		GFile *stream = g_file_new_for_uri (FileName.c_str ());
+		GOutputStream *output = G_OUTPUT_STREAM (g_file_create (stream, G_FILE_CREATE_NONE, NULL, &error));
+		if (error) {
+			cerr << "gio error: " << error->message << endl;
+			g_error_free (error);
+			g_object_unref (file);
+			throw (int) 1;
+		}
 		std::map<std::string, sAtom>AtomsMap;
 		std::map<std::string, sLine>LinesMap;
-		if ((res = gnome_vfs_create (&handle, FileName.c_str (), GNOME_VFS_OPEN_WRITE, true, 0644)) != GNOME_VFS_OK)
-			throw (int) res;
 		old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
 		setlocale(LC_NUMERIC, "C");
 
 		file << "#VRML V2.0 utf8" << endl;
 		
 		//Create prototypes for atoms
-		CrystalAtomList::iterator i;
+		CrystalAtomList::const_iterator i;
 		for (i = Atoms.begin(); i != Atoms.end(); i++)
 		{
 			(*i)->GetColor(&x0, &x1, &x2, &x3);
@@ -459,7 +612,7 @@ void gcDocument::OnExportVRML (const string &FileName)
 		}
 	
 		//Create prototypes for bonds
-		CrystalLineList::iterator j;
+		CrystalLineList::const_iterator j;
 		n = 0;
 		for (j = Lines.begin(); j != Lines.end(); j++)
 		{
@@ -539,12 +692,18 @@ void gcDocument::OnExportVRML (const string &FileName)
 
 		setlocale(LC_NUMERIC, old_num_locale);
 		g_free(old_num_locale);
-		if ((res = gnome_vfs_write (handle, file.str ().c_str (), (GnomeVFSFileSize) file.str ().size (), &fs)) != GNOME_VFS_OK)
-			throw (int) res;
-		gnome_vfs_close (handle);
+		g_output_stream_write (output, file.str ().c_str (), file.str ().size (), NULL, &error);
+		if (error) {
+			cerr << "gio error: " << error->message << endl;
+			g_error_free (error);
+			g_object_unref (stream);
+			throw (int) 1;
+		}
+		g_output_stream_close (output, NULL, NULL);
+		g_object_unref (stream);
 	}
 	catch (int n) {
-		fprintf (stderr, "gnome-vfs error #%d\n",n);
+		// TODO: implement a meaningful error handler.
 	}
 }
 
@@ -592,7 +751,7 @@ bool gcDocument::VerifySaved()
 	m_bClosing = true;
 	if (!GetDirty ())
 		return true;
-	gchar* str = g_strdup_printf(_("\"%s\" has been modified.  Do you wish to save it?"), m_title);
+	gchar* str = g_strdup_printf(_("\"%s\" has been modified.  Do you wish to save it?"), m_Label);
 	GtkWidget* mbox;
 	int res;
 	do
@@ -639,7 +798,7 @@ CrystalCleavage* gcDocument::CreateNewCleavage()
 	return (CrystalCleavage*) new gcCleavage();
 }
 
-const char* gcDocument::GetProgramId()
+const char* gcDocument::GetProgramId () const
 {
 	return "Gnome Crystal "VERSION;
 }
@@ -669,12 +828,131 @@ void gcDocument::RenameViews ()
 		if (!w)
 			continue;
 		if (max > 1) {
-			char *t = g_strdup_printf ("%s (%i)", m_title, n++);
+			char *t = g_strdup_printf ("%s (%i)", m_Label, n++);
 			gtk_window_set_title (w, t);
 			g_free (t);
 		} else
-			gtk_window_set_title (w, m_title);
+			gtk_window_set_title (w, m_Label);
 		window->ActivateActionWidget ("ui/MainMenu/FileMenu/Save", !m_ReadOnly);
 		window->ActivateActionWidget ("ui/MainToolbar/Save", !m_ReadOnly);
 	}
 }
+
+#ifdef HAVE_OPENBABEL_2_2
+bool gcDocument::Import (const string &filename, const string& mime_type)
+{
+	gchar *oldfilename, *oldtitle;
+	if (m_filename)
+		oldfilename = g_strdup (m_filename);
+	else oldfilename = NULL;
+	oldtitle = g_strdup (m_title);
+	char *old_num_locale;
+	bool result = false, read_only = false;
+	GFile *file;
+	GFileInfo *info = NULL;
+	GError *error = NULL;
+	try {
+		if (!filename.length ())
+			throw (int) 0;
+		file = g_file_new_for_uri (filename.c_str ());
+		info = g_file_query_info (file,
+								  G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE","G_FILE_ATTRIBUTE_STANDARD_SIZE,
+								  G_FILE_QUERY_INFO_NONE, NULL, &error);
+		if (error) {
+			g_warning ("GIO error: %s", error->message);
+			g_error_free (error);
+			if (info)
+				g_object_unref (info);
+			g_object_unref (file);
+			throw (int) 1;
+		}
+		gsize size = g_file_info_get_size (info);
+		read_only = !g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
+		g_object_unref (info);
+		if (SetFileName (filename), !m_filename)
+			throw (int) 2;
+		g_free (oldfilename);
+		g_free (oldtitle);
+		char *buf = new char[size + 1];
+		GInputStream *input = G_INPUT_STREAM (g_file_read (file, NULL, &error));
+		gsize n = 0;
+		while (n < size) {
+			n += g_input_stream_read (input, buf, size, NULL, &error);
+			if (error) {
+				g_message ("GIO could not read the file: %s", error->message);
+				g_error_free (error);
+				delete [] buf;
+				g_object_unref (input);
+				g_object_unref (file);
+				throw (int) 3;
+			}
+		}
+		g_object_unref (input);
+		g_object_unref (file);
+		buf[size] = 0;
+		istringstream iss (buf);
+		old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+		setlocale(LC_NUMERIC, "C");
+		OBMol Mol;
+		OBConversion Conv;
+		OBFormat* pInFormat = OBFormat::FormatFromMIME (mime_type.c_str ());
+		if (pInFormat == NULL)
+			throw (int) 4;
+		Conv.SetInFormat (pInFormat);
+		Conv.Read (&Mol, &iss);
+		result = ImportOB (Mol);
+		Mol.Clear ();
+		setlocale (LC_NUMERIC, old_num_locale);
+		g_free (old_num_locale);
+		g_free (buf);
+		if (!result)
+			throw (int) 5;
+		UpdateAllViews ();
+		return true;
+	}
+	catch (int num) {
+		switch (num)
+		{
+		default:
+			Error(LOAD);
+		}
+		if (num >= 0) {
+			if (oldfilename)  {
+				SetFileName (oldfilename);
+				g_free (oldfilename);
+			} else {
+				g_free (m_filename);
+				m_filename = NULL;
+			}
+			SetTitle (oldtitle);
+			g_free (oldtitle);
+		}
+		return false;
+	}
+	return false;
+}
+#endif
+
+void gcDocument::SetAuthor (char const *author)
+{
+	g_free (m_Author);
+	m_Author = g_strdup (author);
+}
+
+void gcDocument::SetMail (char const *mail)
+{
+	g_free (m_Mail);
+	m_Mail = g_strdup (mail);
+}
+
+void gcDocument::SetComment (char const *comment)
+{
+	g_free (m_Comment);
+	m_Comment = g_strdup (comment);
+}
+
+void gcDocument::SetLabel (char const *label)
+{
+	g_free (m_Label);
+	m_Label = g_strdup (label);
+}
